<?php
/**
 * Sortablejs: Javascript for Sortable table
 *
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Otto Vainio
 * version 1.1 Fixed javascript error in sorttable js
 * version 2.0 Added <div> to sort any table
 * version 2.1 Changed script to allow multiple sortable tables in one page
 * version 2.2 A table can now be sorted by one column by default.
 * version 2.2a css+js compress broke this script. Now fixed some jslint complains.
 * version 2.3 Added support for odt plugin. (Aurlien Bompard)
 */
// must be run within Dokuwiki
if (!defined('DOKU_INC')) die();
if (!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN', DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'syntax.php');

class syntax_plugin_sortablejs extends DokuWiki_Syntax_Plugin {

  function getInfo(){
    return array(
      'author' => 'Otto Vainio',
      'email'  => 'plugins@valjakko.net',
      'date'   => '2009-08-24',
      'name'   => 'Sortable javascript',
      'desc'   => 'Add <sortable>  and </sortable> around your table.',
      'url'    => 'http://www.dokuwiki.org/plugin:sortablejs',
    );
  }
  function getType() { return 'container';}
  function getPType(){ return 'normal';}
  function getSort() { return 999; }
	function connectTo($mode) {
    $this->Lexer->addEntryPattern('<sortable[^>]*>(?=.*?\x3C/sortable\x3E)',$mode,'plugin_sortablejs');
//    $this->Lexer->addEntryPattern('\x3Csortable.*?\x3E',$mode,'plugin_sortablejs');
//    $this->Lexer->addEntryPattern('<sortable>',$mode,'plugin_sortablejs');
  }
  function postConnect() {
    $this->Lexer->addExitPattern('</sortable>','plugin_sortablejs');
  }
  function handle($match, $state, $pos, &$handler){
    
    switch ($state) {
      case DOKU_LEXER_ENTER :
        $match = substr($match,9,-1);
        $match=trim($match);
        $scl="";
        if (strlen($match)>0) {
          $scl=" sort$match";
        }
//        return "<div class=\"sortable$scl\">";
        return array($state, $scl);
        break;
      case DOKU_LEXER_UNMATCHED :
//        return p_render('xhtml',p_get_instructions($match),$info);
        return array($state, $match);
        break;
      case DOKU_LEXER_EXIT :
//        return "</div>";
        return array($state, "");
        break;
    }
    return array();
  }

  function render($mode, &$renderer, $data) {
    list($state,$match) = $data;
    if ($mode == 'xhtml'){
      switch ($state) {
        case DOKU_LEXER_ENTER :
          $renderer->doc .= "<div class=\"sortable$match\">";
          break;
        case DOKU_LEXER_UNMATCHED :
          $renderer->doc .= p_render('xhtml',p_get_instructions($match),$info);
          break;
        case DOKU_LEXER_EXIT :
          $renderer->doc .=  "</div>";
          break;
      }
      return true;
    } else if($mode == 'odt'){
      switch ($state) {
        case DOKU_LEXER_ENTER :
          // In ODT, tables must not be inside a paragraph. Make sure we
          // closed any opened paragraph
          $renderer->p_close();
          break;
        case DOKU_LEXER_UNMATCHED :
          $instructions = array_slice(p_get_instructions($match), 1, -1);
          foreach ($instructions as $instruction) {
            call_user_func_array(array(&$renderer, $instruction[0]),$instruction[1]);
          }
          break;
        case DOKU_LEXER_EXIT :
          $renderer->p_open(); // re-open the paragraph
          break;
      }
      return true;
    }
    return false;
  }
}
?>